% The code is used for 3D reconstruction of EM slices 
% AZ is defined as longer than 5 dots (artificially defined)
% '-d' in the file name if there are docked vesicles


clear;
folder = "E:\Changliang\3D_EM_plot\S11_sy_5";
az_resolution = 1:1:30;  % resample az dots
psd_resolution = 1:1:30;
z_step = 50; % slice thickness
plot_az_check = 1; % determine whether plot AZ 1 yes, 0 no
plot_psd_check = 0;  % determine whether plot PSD 1 yes, 0 no
mesh_thickness = 0.1; % thickness of mesh
vesicle_size = 180; % vesicle size


    
%%

filepattern = fullfile(folder, '*.mat'); % find quantification files
file = dir(filepattern);
figure;
hold;
az_n = 0;
psd_n = 0;
for i = 1:length(file)
    name = strcat(folder,'\', file(i).name);
    load(name);
    vesicle_x = controle.vesicle.x;
    vesicle_y = controle.vesicle.y;
    vesicle_z(1:length(vesicle_x)) = i*50;
    scatter3(vesicle_x,vesicle_y,vesicle_z, 'MarkerEdgeColor', '0.65,0.65,0.65',...
        'MarkerFaceColor', '0.95,0.95,0.95',  'SizeData', vesicle_size, 'MarkerFaceAlpha',.4,'MarkerEdgeAlpha',.8);
       
    if length(controle.az.x)>5  
        az_n = az_n + 1;
        az_x(az_n,:) = interp1(1:length(controle.az.x), controle.az.x, az_resolution);
        az_y(az_n,:) = interp1(1:length(controle.az.y), controle.az.y, az_resolution);
        az_z(az_n,1:length(az_x(az_n,:))) = i*z_step;
        psd_n = psd_n + 1;
        psd_x(psd_n,:) = interp1(1:length(controle.psd.x), controle.psd.x, psd_resolution);
        psd_y(psd_n,:) = interp1(1:length(controle.psd.y), controle.psd.y, psd_resolution);
        psd_z(psd_n,1:length(psd_x(az_n,:))) = i*z_step;
        
        if contains(name,'_d')  % plot the docked vesicle
            dv_x = controle.vesicle_docked.x;
            dv_y = controle.vesicle_docked.y;
            dv_z(1:length(dv_x)) = i*50;
            scatter3(dv_x, dv_y, dv_z, 'MarkerEdgeColor', [0.8500 0.3250 0.0980],...
        'MarkerFaceColor', [0.8500 0.3250 0.0980],  'SizeData', vesicle_size);
        end        
        clear dv_x dv_y dv_z
    end   
    clear vesicle_x vesicle_y vesicle_z 
end


%% modify the data
for i = 1: size(az_x,1) % remove Nan in the az matrix
    for j = 1: size(az_x,2)
        if isnan(az_x(i,j))
            az_x(i,j) = az_x(i,j-1);
        end
        if isnan(az_y(i,j))
            az_y(i,j) = az_y(i,j-1);
        end
    end
end

for i = 1: size(psd_x,1) % remove Nan in the az matrix
    for j = 1: size(psd_x,2)
        if isnan(psd_x(i,j))
            psd_x(i,j) = psd_x(i,j-1);
        end
        if isnan(psd_y(i,j))
            psd_y(i,j) = psd_y(i,j-1);
        end
    end
end

%% Plot az and psd
if plot_az_check == 1
    s_az = surf(az_x, az_y, az_z, 'FaceAlpha', 0.2, 'FaceColor', [0.3010 0.7450 0.9330],'EdgeColor', 'none');
    %colormap([0  0  0]);
    %set(s_az, 'LineWidth', mesh_thickness)
end

        
if plot_psd_check == 1
    s_psd = surf(psd_x, psd_y, psd_z, 'FaceAlpha', 0.2, 'FaceColor', [0.4940 0.1840 0.5560],'EdgeColor', 'none');
    %colormap([0  0  0]);
    %set(s_psd, 'LineWidth', mesh_thickness)
end



%% other methods tried
% resolution = 10;
% n = 0;
% for k = 1: size(az_x,1) - 1
%      for j = 0:resolution-1
%         n = n+1;
%         f_az_x(n,:) = az_x(k,:) + (az_x(k+1,:)-az_x(k,:))/resolution*j;
%         f_az_y(n,:) = az_y(k,:) + (az_y(k+1,:)-az_y(k,:))/resolution*j;
%         f_az_z(n,:) = az_z(k,:) + 50/resolution*j;
%       
%     end
% end

%surf(f_az_x, f_az_y, f_az_z,'FaceAlpha',0.5 )
%%

